#include <Wt/WApplication.h>
#include <Wt/WBreak.h>
#include <Wt/WContainerWidget.h>
#include <Wt/WLocale.h>
#include <Wt/WLocalizedStrings.h>
#include <Wt/WPushButton.h>
#include <Wt/WText.h>

#include <memory>

class LocalizedStrings final : public Wt::WLocalizedStrings {
public:
  Wt::LocalizedString resolveKey(const Wt::WLocale &locale, const std::string &key) override
  {
    if (key == "lang") {
      if (locale.name() == "nl") {
        return {"Nederlands {1}", Wt::TextFormat::XHTML};
      } else {
        return {"English {1}", Wt::TextFormat::XHTML};
      }
    } else {
      return {};
    }
  }
};

class Application final : public Wt::WApplication {
public:
  explicit Application(const Wt::WEnvironment &env)
    : Wt::WApplication(env)
  {
    setLocalizedStrings(std::make_shared<LocalizedStrings>());

    setLocale(Wt::WLocale("en"));

    root()->addNew<Wt::WText>(Wt::utf8(
            "When the button is clicked, the locale changes. You can see\n"
            "that the button is updated properly, but the text isn't."));

    root()->addNew<Wt::WBreak>();
    root()->addNew<Wt::WBreak>();

    auto btn = root()->addNew<Wt::WPushButton>(Wt::WString::tr("lang").arg(1));

    root()->addNew<Wt::WBreak>();
    root()->addNew<Wt::WBreak>();

    root()->addNew<Wt::WText>(Wt::WString::tr("lang").arg(1), Wt::TextFormat::XHTML);

    btn->clicked().connect([this]{
      setLocale(Wt::WLocale(locale().name() == "en" ? "nl" : "en"));
    });
  }
};

int main(int argc, char *argv[])
{
  return Wt::WRun(argc, argv, [](const Wt::WEnvironment &env) {
    return std::make_unique<Application>(env);
  });
}
